@@ -1,6 +1,6 @@
 import numpy as np
 import neighborhood_analysis as na
-from neighborhood_analysis import CellCombs, get_neighbors
+from neighborhood_analysis import CellCombs, get_neighbors, comb_bootstrap
 
 from time import time
 
@@ -12,8 +12,13 @@ neighbors = get_neighbors(points, 10.0)
 
 start = time()
 cc = CellCombs(types)
-cc.bootstrap(corr_types, neighbors)
+cc.bootstrap(corr_types, neighbors, ignore_self=True)
 end = time()
 print(f"used {(end-start):.2f}s")
 
-
+s1 = time()
+X = [bool(i) for i in np.random.choice([True, False], 10000)]
+Y = [bool(i) for i in np.random.choice([True, False], 10000)]
+v = comb_bootstrap(X, Y, neighbors, ignore_self=True)
+s2 = time()
+print(f"used {(s2-s1):.2f}s")
